import os
import shutil
from click.testing import CliRunner


def before_scenario(context, _):
    context.test_dir = os.path.join(os.path.dirname(__file__), 'test_qa')
    context.runner = CliRunner()
    context.command = ['-r', context.test_dir, '--no-editor', '--report-dir', context.test_dir]


def after_scenario(context, _):
    if os.path.exists(context.test_dir):
        shutil.rmtree(context.test_dir)



# -*- coding: utf-8 -*-

import os
import yaml
import mako.template
from nitpicker import helpers


class ReportGenerator:
    SUPPORTED_FORMATS = ['md']

    def __init__(self, report_format='md'):
        if report_format not in self.SUPPORTED_FORMATS:
            raise "Format {} is not supported.".format(report_format)

        self.__template_path = os.path.abspath(
            os.path.join(os.path.dirname(__file__), 'REPORT_TEMPLATE.' + report_format))
        self.__format = report_format

    def generate(self, root, report_name='QA_REPORT', report_dir=''):
        templ_dict = dict()
        templ_dict['header'] = 'QA Report'
        templ_dict['generated_at'] = helpers.get_current_time_as_str()
        templ_dict['qa_dir'] = root
        templ_dict['plans'] = list()

        for root, _, files in os.walk(root):
            files = [f for f in files if '.yml' in f]
            if len(files) == 0:
                continue

            plan = dict()
            plan['name'] = root.replace('\\', '.')
            plan['cases'] = list()

            for case_file in files:
                case = dict()

                with open(os.path.join(root, case_file), encoding='utf-8') as template_file:
                    case_yaml = yaml.load(template_file)

                run_count = 0
                run_dir = os.path.join(root, 'runs')
                last_run = None
                if os.path.exists(run_dir):
                    for run_file in os.listdir(run_dir):

                        with open(os.path.join(run_dir, run_file), encoding='utf-8') as template_file:
                            run_yaml = yaml.load(template_file)

                        if case_file in run_yaml['cases'] \
                                and not run_yaml['cases'][case_file]['status'] == 'skipped':
                            run_count = run_count + 1
                            last_run = run_yaml['cases'][case_file]['finished']

                case['name'] = case_file
                case['desc'] = case_yaml['description']
                case['author'] = case_yaml['author']
                case['count'] = run_count
                case['last_run'] = last_run

                plan['cases'].append(case)

            templ_dict['plans'].append(plan)

        with open(self.__template_path, encoding='utf-8') as template_file:
            templ = mako.template.Template(template_file.read())

        with open(os.path.join(report_dir, '.'.join([report_name, self.__format])), 'w', encoding='utf-8') as report_file:
            report_file.write(templ.render(**templ_dict))




# -*- coding: utf-8 -*-

import click
import os
import yaml
import time
from nitpicker import helpers
from nitpicker.report_generator.generator import ReportGenerator


TEST_CASE_TEMPLATE = '''
created: {created}
author: {author}
description: {description}
tags:
setup:
- Do something to start
actions:
- Action 1
- Action 2
reactions:
- Reaction 1
- Reaction 2
teardown:
- Do something to stop
'''


@click.group()
@click.option('--root', '-r', type=str, default='qa')
@click.option('--no-editor', type=bool, default=False, is_flag=True)
@click.option('--report-dir', default='')
@click.pass_context
def main(ctx, root, no_editor, report_dir):
    ctx.obj = dict()
    ctx.obj['ROOT'] = root
    ctx.obj['NO_EDITOR'] = no_editor
    ctx.obj['REPORT_DIR'] = report_dir

@main.command()
@click.argument('test_case_name')
@click.option('--plan', '-p', type=str, default='',
              help='Select the test plane in the plan tree separated by dot. Example: feature_1.plan_2')
@click.option('--force', '-f', type=bool, default=False, is_flag=True,
              help='Replace the old test case with a new one, if it has the same name.')
@click.pass_context
def add(ctx, test_case_name, plan, force):
    """
    Add a new test case to a plan.
    """
    case_dir = os.path.join(*([ctx.obj['ROOT']] + plan.split('.')))
    case_file_path = os.path.join(case_dir, test_case_name + '.yml')

    if not os.path.exists(case_dir):
        os.makedirs(case_dir)

    if not force and os.path.exists(case_file_path):
        click.echo('Test case "{}" is already created'.format(test_case_name))
        exit(1)

    data = dict()
    data['created'] = helpers.get_current_time_as_str()
    data['author'] = 'Unknown'
    data['description'] = ''

    text = TEST_CASE_TEMPLATE.format(**data)
    if not ctx.obj['NO_EDITOR']:
        text = click.edit(text, extension='.yml')

    f = open(case_file_path, 'w')
    f.write(text)


@main.command()
@click.pass_context
def list(ctx):

    def calc_plans(path):
        count = 0
        for _, dirs_, files_ in os.walk(path):
            files_ = [f for f in files_ if '.yml' in f]
            count += len(files_)

        return count

    click.echo('You project has {} test cases'.format(calc_plans(ctx.obj['ROOT'])))
    for root, dirs, files in os.walk(ctx.obj['ROOT']):
        if not root == ctx.obj['ROOT']:
            level = root.replace(ctx.obj['ROOT'], '').count(os.sep) - 1
            indent = ' '*2*level
            subindent = ' '*2*(level + 1)

            files = [f for f in files if '.yml' in f]
            click.echo('{}Plan "{}" has {} cases:'.format(indent, os.path.basename(root), calc_plans(root)))

            for f in files:
                data = yaml.load(open(os.path.join(root, f), encoding='utf-8'))
                click.echo('{}{} - {}'.format(subindent, f[0:-4], data['description'] if 'description' in data else ''))


@main.command()
@click.argument('test_plan')
@click.pass_context
def run(ctx, test_plan):
    """
    Run a test plan in the plan tree separated by dot
    """
    case_dir = os.path.join(*([ctx.obj['ROOT']] + test_plan.split('.')))

    for root, _, files in os.walk(case_dir):
        files = [f for f in files if '.yml' in f]
        if len(files) == 0:
            continue

        report = dict()
        report['started'] = helpers.get_current_time_as_str()
        report['cases'] = dict()

        for f in files:
            with open(os.path.join(root, f), encoding='utf-8') as case_file:
                data = yaml.load(case_file)

            click.echo('Start test {} - {}? [Y/n]'.format(f, data['description']))

            report['cases'][f] = dict()
            report['cases'][f]['description'] = data['description']
            report['cases'][f]['started'] = helpers.get_current_time_as_str()

            answer = input().strip().lower()
            if answer == 'n':
                click.secho('SKIPPED', fg='yellow')
                report['cases'][f]['status'] = 'skipped'
                continue

            click.echo('You should do this before run the case: \n {} \n '.format('\n'.join(data['setup'])))

            step = 0
            for action, reaction in zip(data['actions'], data['reactions']):
                step += 1
                click.echo('Step {}: \n ACTION: {} \n REACTION: {}\n Is it OK? [Y/n]'
                               .format(step, action, reaction))

                answer = input().strip().lower()
                if answer == 'n':
                    click.secho('FAILED', fg='red')
                    report['cases'][f]['comment'] = input('Comment please the failed step.').strip()
                    report['cases'][f]['status'] = 'failed'
                    report['cases'][f]['failed_step'] = step
                    report['cases'][f]['failed_action'] = action
                    report['cases'][f]['failed_reaction'] = reaction
                    report['cases'][f]['finished'] = helpers.get_current_time_as_str()
                    break
                else:
                    click.secho('PASSED', fg='green')

            if 'status' not in report['cases'][f]:
                report['cases'][f]['status'] = 'passed'
                report['cases'][f]['finished'] = helpers.get_current_time_as_str()

        report['finished'] = helpers.get_current_time_as_str()

        run_dir = os.path.join(root, 'runs')
        if not os.path.exists(run_dir):
            os.makedirs(run_dir)

        with open(os.path.join(run_dir, time.strftime("%Y%m%d_%H%M%S", time.gmtime()) + '_run.report'),
                  'w', encoding='utf-8') as report_file:
            yaml.dump(report, report_file, default_flow_style=False)

        ReportGenerator('md').generate(ctx.obj['ROOT'], report_dir=ctx.obj['REPORT_DIR'])


@main.command()
@click.pass_context
def check(ctx):
    for root, _, files in os.walk(ctx.obj['ROOT']):
        files = [f for f in files if '.report' in f]
        if len(files) == 0:
            continue

        last_report_file = open(os.path.join(root, sorted(files)[-1]), encoding='utf-8')
        report = yaml.load(last_report_file)

        for file, case in report['cases'].items():
            if case['status'] == 'failed':
                click.secho('{} ({}) is failed'.format(file, case['description']), fg='red')
                exit(1)



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open('README.rst') as f:
    readme = f.read()

setup(
    name='nitpicker',
    version='0.1.0',
    description='A simple CLI tool for QA',
    long_description=readme,
    author='Aleksey Timin',
    author_email='atimin@gmail.com',
    url='https://github.com/flipback/nitpicker',
    license='MIT',
    packages=find_packages(exclude=('tests', 'docs', 'qa')),
    python_requires='>=3.3',
    install_requires=[
        'click>=6.7',
        'colorama>=0.3.9',
        'pyyaml>=3.13',
        'mako>=1.0'
    ],
)

